@@ -19,6 +19,10 @@ enum {
 	FD_PROFILE,
 	FD_TIME,
 	FD_NET,
+	FD_RATE,
+	FD_COMPRESS,
+	FD_STEADYSTATE,
+	FD_HELPERTHREAD,
 	FD_DEBUG_MAX,
 };
 
