@@ -6,8 +6,16 @@
 
 #include "fio.h"
 
+#define LOG_START_SZ		512
+
 size_t log_info_buf(const char *buf, size_t len)
 {
+	/*
+	 * buf could be NULL (not just "").
+	 */
+	if (!buf)
+		return 0;
+
 	if (is_backend) {
 		size_t ret = fio_server_text_output(FIO_LOG_INFO, buf, len);
 		if (ret != -1)
@@ -21,40 +29,66 @@ size_t log_info_buf(const char *buf, size_t len)
 		return fwrite(buf, len, 1, f_out);
 }
 
-size_t log_valist(const char *str, va_list args)
+static size_t valist_to_buf(char **buffer, const char *fmt, va_list src_args)
 {
-	char buffer[1024];
+	size_t len, cur = LOG_START_SZ;
+	va_list args;
+
+	do {
+		*buffer = calloc(1, cur);
+
+		va_copy(args, src_args);
+		len = vsnprintf(*buffer, cur, fmt, args);
+		va_end(args);
+
+		if (len < cur)
+			break;
+
+		cur = len + 1;
+		free(*buffer);
+	} while (1);
+
+	return len;
+}
+
+size_t log_valist(const char *fmt, va_list args)
+{
+	char *buffer;
 	size_t len;
 
-	len = vsnprintf(buffer, sizeof(buffer), str, args);
+	len = valist_to_buf(&buffer, fmt, args);
+	len = log_info_buf(buffer, len);
+	free(buffer);
 
-	return log_info_buf(buffer, min(len, sizeof(buffer) - 1));
+	return len;
 }
 
 size_t log_info(const char *format, ...)
 {
-	char buffer[1024];
 	va_list args;
-	size_t len;
+	size_t ret;
 
 	va_start(args, format);
-	len = vsnprintf(buffer, sizeof(buffer), format, args);
+	ret = log_valist(format, args);
 	va_end(args);
 
-	return log_info_buf(buffer, min(len, sizeof(buffer) - 1));
+	return ret;
 }
 
 size_t __log_buf(struct buf_output *buf, const char *format, ...)
 {
-	char buffer[1024];
+	char *buffer;
 	va_list args;
 	size_t len;
 
 	va_start(args, format);
-	len = vsnprintf(buffer, sizeof(buffer), format, args);
+	len = valist_to_buf(&buffer, format, args);
 	va_end(args);
 
-	return buf_output_add(buf, buffer, min(len, sizeof(buffer) - 1));
+	len = buf_output_add(buf, buffer, len);
+	free(buffer);
+
+	return len;
 }
 
 int log_info_flush(void)
@@ -67,33 +101,33 @@ int log_info_flush(void)
 
 size_t log_err(const char *format, ...)
 {
-	char buffer[1024];
+	size_t ret, len;
+	char *buffer;
 	va_list args;
-	size_t len;
 
 	va_start(args, format);
-	len = vsnprintf(buffer, sizeof(buffer), format, args);
+	len = valist_to_buf(&buffer, format, args);
 	va_end(args);
-	len = min(len, sizeof(buffer) - 1);
 
 	if (is_backend) {
-		size_t ret = fio_server_text_output(FIO_LOG_ERR, buffer, len);
+		ret = fio_server_text_output(FIO_LOG_ERR, buffer, len);
 		if (ret != -1)
-			return ret;
+			goto done;
 	}
 
 	if (log_syslog) {
 		syslog(LOG_INFO, "%s", buffer);
-		return len;
+		ret = len;
 	} else {
-		if (f_err != stderr) {
-			int fio_unused ret;
-
+		if (f_err != stderr)
 			ret = fwrite(buffer, len, 1, stderr);
-		}
 
-		return fwrite(buffer, len, 1, f_err);
+		ret = fwrite(buffer, len, 1, f_err);
 	}
+
+done:
+	free(buffer);
+	return ret;
 }
 
 const char *log_get_level(int level)
