@@ -15,9 +15,8 @@ enum {
 	SW_F_IDLE	= 1 << 0,
 	SW_F_RUNNING	= 1 << 1,
 	SW_F_EXIT	= 1 << 2,
-	SW_F_EXITED	= 1 << 3,
-	SW_F_ACCOUNTED	= 1 << 4,
-	SW_F_ERROR	= 1 << 5,
+	SW_F_ACCOUNTED	= 1 << 3,
+	SW_F_ERROR	= 1 << 4,
 };
 
 static struct submit_worker *__get_submit_worker(struct workqueue *wq,
@@ -131,7 +130,7 @@ static void *worker_thread(void *data)
 {
 	struct submit_worker *sw = data;
 	struct workqueue *wq = sw->wq;
-	unsigned int eflags = 0, ret = 0;
+	unsigned int ret = 0;
 	FLIST_HEAD(local_list);
 
 	sk_out_assign(sw->sk_out);
@@ -206,9 +205,6 @@ handle_work:
 		wq->ops.update_acct_fn(sw);
 
 done:
-	pthread_mutex_lock(&sw->lock);
-	sw->flags |= (SW_F_EXITED | eflags);
-	pthread_mutex_unlock(&sw->lock);
 	sk_out_drop();
 	return NULL;
 }
@@ -278,8 +274,11 @@ static int start_worker(struct workqueue *wq, unsigned int index,
 	int ret;
 
 	INIT_FLIST_HEAD(&sw->work_list);
-	pthread_cond_init(&sw->cond, NULL);
-	pthread_mutex_init(&sw->lock, NULL);
+
+	ret = mutex_cond_init_pshared(&sw->lock, &sw->cond);
+	if (ret)
+		return ret;
+
 	sw->wq = wq;
 	sw->index = index;
 	sw->sk_out = sk_out;
@@ -308,17 +307,24 @@ int workqueue_init(struct thread_data *td, struct workqueue *wq,
 {
 	unsigned int running;
 	int i, error;
+	int ret;
 
 	wq->max_workers = max_workers;
 	wq->td = td;
 	wq->ops = *ops;
 	wq->work_seq = 0;
 	wq->next_free_worker = 0;
-	pthread_cond_init(&wq->flush_cond, NULL);
-	pthread_mutex_init(&wq->flush_lock, NULL);
-	pthread_mutex_init(&wq->stat_lock, NULL);
+
+	ret = mutex_cond_init_pshared(&wq->flush_lock, &wq->flush_cond);
+	if (ret)
+		goto err;
+	ret = mutex_init_pshared(&wq->stat_lock);
+	if (ret)
+		goto err;
 
 	wq->workers = smalloc(wq->max_workers * sizeof(struct submit_worker));
+	if (!wq->workers)
+		goto err;
 
 	for (i = 0; i < wq->max_workers; i++)
 		if (start_worker(wq, i, sk_out))
